BAD_CONFIG_FILE
	"nemohu pochopit obsah souboru %s\n"
CONFIG_OPEN_ERROR
	"Varování: nemohu otevřít konfigurační soubor %s\n"
PARSE_ERROR_IN_CONFIG
	"Chyba v analýze konfiguračního souboru\n"
INCOMPAT
	"nekompatibilní volby %s a %s\n"
NO_ALTERNATE
	"Promiňte, ale podpora pro jiné systémy nebyla zkompilována\n"
NO_COMPRESS
	"Man byl konfigurován pro automatickou kompresi formátovaných stránek,\n\
ale konfigurační soubor nedefinuje COMPRESS.\n"
NO_NAME_FROM_SECTION
	"Kterou manuálovou stránku ze sekce %s chcete?\n"
NO_NAME_NO_SECTION
	"Kterou manuálovou stránku chcete?\n"
NO_SUCH_ENTRY_IN_SECTION
	"Žádný záznam o stránce %s v sekci %s manuálu\n"
NO_SUCH_ENTRY
	"Žádný záznam pro %s\n"
PAGER_IS
	"\npoužívám %s jako stránkovač\n"
SYSTEM_FAILED
	"Nastala chyba při spuštění formátovacího nebo zobrazovacího příkazu.\n\
Příkaz %s skončil a vrátil chybovou hodnotu %d.\n"
VERSION
	"%s, verze %s\n\n"
OUT_OF_MEMORY
	"Nedostatek paměti - nemohu alokovat %d bytů\n"
ROFF_CMD_FROM_FILE_ERROR
	"Chyba při analýze *roff příkazu v souboru %s\n"
MANROFFSEQ_ERROR
	"Chybná analýza MANROFFSEQ.  Použiji systémový standard.\n"
ROFF_CMD_FROM_COMMANDLINE_ERROR
	"Chyba při analýze *roff příkazu z příkazové řádky.\n"
UNRECOGNIZED_LINE
	"Nerozpoznal jsem řádek v konfiguračním souboru (ignoruji jej)\n%s\n"
GETVAL_ERROR
	"man-config.c: vnitřní chyba: řetězec %s nebyl nalezen\n"
FOUND_MANDIR
	"našel jsem manuálový adresář %s\n"
FOUND_MAP
	"nalezená manuálová cesta je mapována jako %s --> %s\n"
FOUND_CATDIR
	"odpovídající adresář zformátovaných stránek je %s\n"
LINE_TOO_LONG
	"Příliš dlouhý řádek v konfiguračním souboru\n"
SECTION
	"\nsekce: %s\n"
UNLINKED
	"nespojeno %s\n"
GLOBBING
	"provádím nahrazování %s\n"
EXPANSION_FAILED
	"Pokus [%s] o formátování manuálové stránky selhal\n"
OPEN_ERROR
	"Nemohu otevřít manuálovou stránku %s\n"
READ_ERROR
	"Chyba při čtení manuálové stránky %s\n"
FOUND_EQN
	"nalezen příkaz pro eqn(1)\n"
FOUND_GRAP
	"nalezen příkaz pro grap(1)\n"
FOUND_PIC
	"nalezen příkaz pro pic(1)\n"
FOUND_TBL
	"nalezen příkaz pro tbl(1)\n"
FOUND_VGRIND
	"nalezen příkaz pro vgrind(1)\n"
FOUND_REFER
	"nalezen příkaz pro refer(1)\n"
ROFF_FROM_COMMAND_LINE
	"analyzuji příkazy z příkazové řádky\n"
ROFF_FROM_FILE
	"analyzuji příkazy ze souboru %s\n"
ROFF_FROM_ENV
	"analyzuji příkazy z prostředí\n"
USING_DEFAULT
	"používám standardní volání preprocesoru\n"
PLEASE_WAIT
	"Formátuji stránku, prosím počkejte...\n"
CHANGED_MODE
	"změněn mód z %s na %o\n"
CAT_OPEN_ERROR
	"Nemohu otevřít %s pro zápis.\n"
PROPOSED_CATFILE
	"bude-li třeba zkusím zapisovat do %s\n"
IS_NEWER_RESULT
	"návratová hodnota z is_newer() = %d\n"
TRYING_SECTION
	"zkouším sekci %s\n"
SEARCHING
	"\nhledám v %s\n"
ALREADY_IN_MANPATH
	"ale %s už je v manuálové cestě\n"
CANNOT_STAT
	"Varování: nemohu nalézt soubor %s!\n"
IS_NO_DIR
	"Varování: %s není adresář!\n"
ADDING_TO_MANPATH
	"přidávám %s do manuálové cesty\n"
PATH_DIR
	"\nadresář cesty %s "
IS_IN_CONFIG
	"je v konfiguračním souboru\n"
IS_NOT_IN_CONFIG
	"není v konfiguračním souboru\n"
MAN_NEARBY
	"ale poblíž je manuálový adresář\n"
NO_MAN_NEARBY
	"a poblíž jsem nenalezl žádný manuálový adresář\n"
ADDING_MANDIRS
	"\npřidávám povinný manuálový adresář\n\n"
CATNAME_IS
	"cat_name v convert_to_cat () je: %s\n"
NO_EXEC
	"\nnespouštím příkaz:\n  %s\n"
USAGE1
	"použití: %s [-adfhktwW] [sekce] [-M cesta] [-P stránkovač] [-S seznam]\n\t"
USAGE2
	"[-m systém] "
USAGE3
	"[-p řetězec] jméno ...\n\n"
USAGE4
	"  a : najdi všechny odpovídající stránky\n\
  c : nepoužívej již zformátovanou stránku\n\
  d : tiskni ladící informace\n\
  D : jako -d, ale zobraz také manuálovou stránku\n\
  f : totéž jako whatis(1)\n\
  h : vytiskni tuto pomocnou zprávu\n\
  k : totéž jako apropos(1)\n\
  K : hledej řetězec ve všech stránkách\n"
USAGE5
	"  t : ke zformátování použij program troff\n"
USAGE6
	"\
  w : vytiskni umístění manuálových stránek, které by byly zobrazeny\n\
      (není-li uvedeno žádné jméno, zobraz prohledávané adresáře)\n\
  W : jako -w, ale zobraz pouze jména soubor;\n\n\
  C soubor     : použij `soubor' jako konfigurační soubor\n\
  M cesta      : nastav manuálové cesty na `cesta'\n\
  P stránkovač : použij program `stránkovač' k zobrazení stránek\n\
  S seznam     : čárkou oddělený seznam manuálových sekcí\n"
USAGE7
	"  m systém : hledej manuálové stránky jiného systému\n"
USAGE8
	"  p řetězec : řetězec určuje preprocesor, který bude spuštěn\n\
               e - [n]eqn(1)   p - pic(1)    t - tbl(1)\n\
               g - grap(1)     r - refer(1)  v - vgrind(1)\n"
USER_CANNOT_OPEN_CAT
	"a skutečný uživatel také nemůže otevřít již zformátovanou stránku\n"
USER_CAN_OPEN_CAT
	"ale skutečný uživatel může otevřít již zformátovanou stránku\n"
CANNOT_FORK
	"nemohu vytvořit další proces pro _%s_\n"
WAIT_FAILED
	"chyba při čekání na potomka _%s_\n"
GOT_WRONG_PID
	"velmi podivné ..., dostal jsem špatné PID při čekání na potomka\n"
CHILD_TERMINATED_ABNORMALLY
	"fatálí chyba: příkaz _%s_ byl abnormálně ukončen\n"
IDENTICAL
	"Manuálová stránka %s je shodná s %s\n"
MAN_FOUND
	"Nalezená(é) manuálová(é) stránky:\n"
NO_TROFF
	"chyba: příkaz TROFF nebyl specifikován v %s\n"
NO_CAT_FOR_NONSTD_LL
	"žádná zformátovaná stránka nebyla uložena (nestandardní délka řádku)\n"
