BAD_CONFIG_FILE
	"nie mogę zrozumieć zawartości pliku %s\n"
CONFIG_OPEN_ERROR
	"Ostrzeżenie: nie mogę otworzyć pliku %s\n"
PARSE_ERROR_IN_CONFIG
	"Błąd przy interpretowaniu pliku konfiguracji\n"
INCOMPAT
	"nie pasujące wzajemnie opcje %s i %s\n"
NO_ALTERNATE
	"Niestety - nie mam wkompilowanej możliwości użycia alternatywnych systemów\n"
NO_COMPRESS
	"Man został skompilowany z automatyczną kompresją stron cat,\n\
ale plik konfiguracyjny nie definiuje COMPRESS.\n"
NO_NAME_FROM_SECTION
	"Jaką stronę w sekcji %s chcesz przeczytać?\n"
NO_NAME_NO_SECTION
	"Jaką stronę chcesz przeczytać?"
NO_SUCH_ENTRY_IN_SECTION
	"Nie ma strony %s w sekcji %s manuala\n"
NO_SUCH_ENTRY
	"Nie ma strony manuala dla %s\n"
PAGER_IS
	"\nuzywam %s jako pagera\n"
SYSTEM_FAILED
	"Błąd w trakcie formatowania lub wyświetlania.\n\
Komenda %s zwróciła status %d.\n"
VERSION
	"%s, wersja %s\n\n"
OUT_OF_MEMORY
	"Brak pamięci - nie mogę malloc %d bajtów\n"
ROFF_CMD_FROM_FILE_ERROR
	"Błąd przetwarzania komendy *roff z plku %s\n"
MANROFFSEQ_ERROR
	"Błąd przetwarzania MANROFFSEQ. Uzyję systemowych wartości domyślnych.\n"
ROFF_CMD_FROM_COMMANDLINE_ERROR
	"Błąd przetwarzania komendy *roff podanej w linii komend.\n"
UNRECOGNIZED_LINE
	"Nierozpoznana linia w pliku konfiguracji (zignorowana)\n%s\n"
GETVAL_ERROR
	"man-config.c: błąd wewnętrzny: nie ma łańcucha %s\n"
FOUND_MANDIR
	"znaleziony katalog man %s\n"
FOUND_MAP
	"znaleziona mapa manpath %s --> %s\n"
FOUND_CATDIR
	"odpowiadający katalog cat: %s\n"
LINE_TOO_LONG
	"Za długa linia w pliku konfiguracyjnym\n"
SECTION
	"\nsekcja: %s\n"
UNLINKED
	"skasowany %s\n"
GLOBBING
	"globbing %s\n"
EXPANSION_FAILED
	"Nieudane usiłowanie [%s] ekspansji strony manuala\n"
OPEN_ERROR
	"Nie mogę otworzyć strony manuala %s\n"
READ_ERROR
	"Błąd czytania strony manuala %s\n"
FOUND_EQN
	"znaleziona dyrektywa eqn(1)\n"
FOUND_GRAP
	"znaleziona dyrektywa grap(1)\n"
FOUND_PIC
	"znaleziona dyrektywa pic(1)\n"
FOUND_TBL
	"znaleziona dyrektywa tbl(1)\n"
FOUND_VGRIND
	"znaleziona dyrektywa vgrind(1)\n"
FOUND_REFER
	"znaleziona dyrektywa refer(1)\n"
ROFF_FROM_COMMAND_LINE
	"przetwarzam dyrektywę podaną w linii komend\n"
ROFF_FROM_FILE
	"przetwarzam dyrektywę z pliku %s\n"
ROFF_FROM_ENV
	"przetwarzam dyrektywę ze zmiennej środowiskowej\n"
USING_DEFAULT
	"używam domyślnej sekwencji preprocesora\n"
PLEASE_WAIT
	"Formatuje stronę, poczekaj...\n"
CHANGED_MODE
	"uprawnienia %s zmienione na %o\n"
CAT_OPEN_ERROR
	"Nie mogę otworzyć %s do zapisu.\n"
PROPOSED_CATFILE
	"będę próbował wpisać do %s, jeżeli potrzebne\n"
IS_NEWER_RESULT
	"status z is_newer() = %d\n"
TRYING_SECTION
	"próbuję sekcję %s\n"
SEARCHING
	"\nszukam w %s\n"
ALREADY_IN_MANPATH
	"ale %s jest już w manpath\n"
CANNOT_STAT
	"Ostrzeżenie: nie mogę użyć stat na pliku %s!\n"
IS_NO_DIR
	"Ostrzeżenie: %s nie jest katalogiem!\n"
ADDING_TO_MANPATH
	"dodaję %s do manpath\n"
PATH_DIR
	"\nkatalog path %s "
IS_IN_CONFIG
	"jest w pliku konfiguracyjnym\n"
IS_NOT_IN_CONFIG
	"nie jest obecny w pliku konfiguracyjnym\n"
MAN_NEARBY
	"ale jest katalog man niedaleko (?)\n"
NO_MAN_NEARBY
	"i nie ma katalogu man niedaleko (?)\n"
ADDING_MANDIRS
	"\ndodaję obowiązkowe katalogi man\n\n"
CATNAME_IS
	"cat_name w convert_to_cat () jest: %s\n"
NO_EXEC
	"\nnie wykonuję komrndy:\n  %s\n"
USAGE1
	"użycie: %s [-adfhktw] [sekcja] [-M ścieżka] [-P pager] [-S lista]\n\t"
USAGE2
	"[-m system] "
USAGE3
	"[-p łańcuch_znaków] nazwa ...\n\n"
USAGE4
	"  a : (all) znajdź wszystkie pasujące strony\n\
  c : (no cat) nie używaj pliku cat\n\
  d : (debug) wyświetlaj mnóstwo informacji o wykonywaniu\n\
  D : jak -d, ale wyświetl też strony man\n\
  f : to samo co whatis(1)\n\
  h : wyświetl ten help\n\
  k : to samo co apropos(1)\n\
  K : szukaj łańcucha znaków we wszystkich stronach\n"
USAGE5
	"  t : użyj troff do formatowania stron do wydrukowania\n"
USAGE6
	"\
  w : wyświetl położenie stron(y) man, która bylaby byla wyświetlona\n\
      (jeżeli nie podana żadna nazwa: wyświetl przeszukiwane katalogi)\n\n\
  W : tak jak -w, ale wypisz tylko nazwy plików\n\n\
  C plik   : uzyj `file' jako plik konfiguracyjny\n\
  M ścieżka: ustaw ścieżkę poszukiwania stron jako `ścieżka'\n\
  P pager  : użyj programu `pager' do wyswietlania stron\n\
  S lista  : lista sekcji oddzielona dwukropkami\n"
USAGE7
	"  m system : szukaj stron man alternatywnego systemu\n"
USAGE8
	"  p łańcuch : łańcuch znaków podający, jakie uzyć prepocesory\n\
               e - [n]eqn(1)   p - pic(1)    t - tbl(1)\n\
               g - grap(1)     r - refer(1)  v - vgrind(1)\n"
USER_CANNOT_OPEN_CAT
	"a rzeczywisty użytkownik też nie może otworzyć pliku\n"
USER_CAN_OPEN_CAT
	"ale rzeczywisty użytkownik może otworzyć plik\n"
CANNOT_FORK
	"neudana próba fork komendy _%s_\n"
WAIT_FAILED
	"błąd w trakcie czekania na proces potomny _%s_\n"
GOT_WRONG_PID
	"bardzo dziwne..., zły pid w trakcie czekania na mój proces potomny\n"
CHILD_TERMINATED_ABNORMALLY
	"błąd krytyczny: komenda _%s_ zakończona nieprawidlowo\n"
IDENTICAL
	"Strona man %s jest identyczna jak %s\n"
MAN_FOUND
	"Znalazłem stronę(y) manuala:\n"
NO_TROFF
	"błąd: nie podana komenda TROFF w %s\n"
NO_CAT_FOR_NONSTD_LL
	"strona cat nie została zapisana z powodu niestandardowej długości linii\n"
